////////////// OpenGL drawtools for 3D Functions ver 1 /////////////////
#include <math.h>
#include <GL/glut.h>
#include "drawtools3D.h"

matrix3D_t createIdentity(void)
{
    matrix3D_t u;
    int i,j;
    for (i=0;i<4;i++) {
        for(j=0;j<4;j++) u.m[i][j]=0.;
        u.m[i][i]=1.;
    }
    return u;
}

matrix3D_t multiply(matrix3D_t a,matrix3D_t b)
{
    matrix3D_t c;//c=a*b
    int i,j,k;
    for (i=0;i<4;i++) for (j=0;j<4;j++) {
        c.m[i][j]=0;
        for (k=0;k<4;k++) c.m[i][j]+=a.m[i][k]*b.m[k][j];
    }
    return c;
}

vector3D_t multiply(matrix3D_t a, vector3D_t b)
{
    vector3D_t c;//c=a*b
    int i,j;
    for (i=0;i<4;i++) {
        c.v[i]=0;
        for (j=0;j<4;j++) c.v[i]+=a.m[i][j]*b.v[j];
    }
    return c;
}

matrix3D_t operator * (matrix3D_t a, matrix3D_t b)
{
    matrix3D_t c;//c=a*b
    int i,j,k;
    for (i=0;i<4;i++) for (j=0;j<4;j++) {
        c.m[i][j]=0;
        for (k=0;k<4;k++) c.m[i][j]+=a.m[i][k]*b.m[k][j];
    }
    return c;
}

vector3D_t operator * (matrix3D_t a, vector3D_t b)
{
    vector3D_t c;//c=a*b
    int i,j;
    for (i=0;i<4;i++) {
        c.v[i]=0;
        for (j=0;j<4;j++) c.v[i]+=a.m[i][j]*b.v[j];
    }
    return c;
}

matrix3D_t translationMTX(float dx,float dy,float dz)
{
    matrix3D_t trans=createIdentity();
    trans.m[0][3]=dx;
    trans.m[1][3]=dy;
    trans.m[2][3]=dz;
    return trans;
}

matrix3D_t rotationXMTX(float theta)
{
    matrix3D_t rotate=createIdentity();
    float cs=cos(theta);
    float sn=sin(theta);
    rotate.m[1][1]=cs; rotate.m[1][2]=-sn;
    rotate.m[2][1]=sn; rotate.m[2][2]=cs;
    return rotate;
}

matrix3D_t rotationYMTX(float theta)
{
    matrix3D_t rotate=createIdentity();
    float cs=cos(theta);
    float sn=sin(theta);
    rotate.m[2][2]=cs; rotate.m[2][0]=-sn;
    rotate.m[0][2]=sn; rotate.m[0][0]=cs;
    return rotate;
}

matrix3D_t rotationZMTX(float theta)
{
    matrix3D_t rotate=createIdentity();
    float cs=cos(theta);
    float sn=sin(theta);
    rotate.m[0][0]=cs; rotate.m[0][1]=-sn;
    rotate.m[1][0]=sn; rotate.m[1][1]=cs;
    return rotate;
}

matrix3D_t scalingMTX(float factorx,float factory,float factorz)
{
    matrix3D_t scale=createIdentity();
    scale.m[0][0]=factorx;
    scale.m[1][1]=factory;
    scale.m[2][2]=factorz;
    return scale;
}

matrix3D_t perspectiveMTX(float eyelength)
{
    matrix3D_t perspective=createIdentity();
    perspective.m[3][2]=-1./eyelength;
    return perspective;
}

vector3D_t Point2Vector(point3D_t pnt)
{
    vector3D_t vec;
    vec.v[0]=pnt.x;
    vec.v[1]=pnt.y;
    vec.v[2]=pnt.z;
    vec.v[3]=1.;
    return vec;
}

point3D_t Vector2Point(vector3D_t vec)
{
    point3D_t pnt;
    pnt.x=vec.v[0];
    pnt.y=vec.v[1];
    pnt.z=vec.v[2];
    return pnt;
}

vector3D_t homogenizeVector(vector3D_t vec)
{
    int i;
    for (i=0;i<3;i++) {
        vec.v[i]/=vec.v[3];
    }
    vec.v[3]=1.;
    return vec;
}

vector3D_t unitVector(vector3D_t vec)
{
    int i;
    float vec2=0.;
    float vec1,invvec1;
    for (i=0;i<3;i++) {
        vec2+=vec.v[i]*vec.v[i];
    }
    vec1=sqrt(vec2);
    if (vec1!=0.) {
        invvec1=1./vec1;
        for (i=0;i<3;i++) {
            vec.v[i]*=invvec1;
        }
    }
    vec.v[3]=1.;
    return vec;
}

// inner product (dot product) of homogeneous vector
float operator * (vector3D_t a, vector3D_t b)
{
    float c;//c=a*b
    int i;
    c=0;
    for (i=0;i<3;i++) {
        c+=a.v[i]*b.v[i];
    }
    return c;
}

// outer product (cross product ) of homogeneous vector
//       i         j         k
//       a0       a1        a2
//       b0       b1        b2
vector3D_t operator ^ (vector3D_t a, vector3D_t b)
{
    vector3D_t c;//c=a*b
    c.v[0]=a.v[1]*b.v[2]-a.v[2]*b.v[1];
    c.v[1]=a.v[2]*b.v[0]-a.v[0]*b.v[2];
    c.v[2]=a.v[0]*b.v[1]-a.v[1]*b.v[0];
    c.v[3]=1.;
    return c;
}

vector3D_t operator - (vector3D_t v1,vector3D_t v0)
{
    vector3D_t c;//c=v1-v0
    c.v[0]=v1.v[0]-v0.v[0];
    c.v[1]=v1.v[1]-v0.v[1];
    c.v[2]=v1.v[2]-v0.v[2];
    c.v[3]=1.;
    return c;
}

vector3D_t operator - (vector3D_t v)
{
    vector3D_t c;//c=-v
    c.v[0]=-v.v[0];
    c.v[1]=-v.v[1];
    c.v[2]=-v.v[2];
    c.v[3]=1.;
    return c;
}

vector3D_t operator * (float r, vector3D_t b)
{
    vector3D_t c;//c=r*b
    int i;
    for (i=0;i<3;i++) {
        c.v[i]=r*b.v[i];
    }
    c.v[3]=1.;
    return c;
}

vector3D_t operator * (vector3D_t b, float r)
{
    vector3D_t c;//c=r*b
    int i;
    for (i=0;i<3;i++) {
        c.v[i]=r*b.v[i];
    }
    c.v[3]=1.;
    return c;
}

float funcPositive(float x)
{
    if (0.<x) return x;
    else return 0.;
}

// x to yth power
float power(float x,float y)
{
    //ln z = y ln x        z = exp (y ln x)
    if (x==0.) return 0;
    return exp(y*log(x));
}

color_t operator + (color_t c1, color_t c2)
{
    color_t col;
    col.r=c1.r+c2.r;
    col.g=c1.g+c2.g;
    col.b=c1.b+c2.b;
    return col;
}

color_t operator * (float r, color_t c)
{
    color_t col;
    col.r=r*c.r;
    col.g=r*c.g;
    col.b=r*c.b;
    return col;
}

color_t operator * (color_t c, float r)
{
    color_t col;
    col.r=r*c.r;
    col.g=r*c.g;
    col.b=r*c.b;
    return col;
}

//PhongModel color calculation
// LightVector, NormalVector, ViewVector, ColorofObject
color_t PhongModel(vector3D_t Light,vector3D_t Normal,vector3D_t View,color_t col)
{
    float kspe=0.7; // specular reflection coefficient
    float kdif=0.6; // diffuse reflection coefficient
    float kamb=0.4; // ambient light coefficient
    float tmp,NL,RV;
    color_t ColWhite={1,1,1};
    vector3D_t ReflectionVector=(2.*(Light*Normal)*Normal)-Light;
    tmp=Normal*Light;
    NL=funcPositive(tmp);
    tmp=ReflectionVector*View;
    RV=funcPositive(tmp);
    return kdif*NL*col+kspe*power(RV,4)*ColWhite+kamb*col;
}

void setColor(color_t col)
{
    glColor3f(col.r, col.g, col.b); 
}

void drawDot(point3D_t pt)
{
    glBegin(GL_POINTS);
        glVertex2f(pt.x, pt.y);
    glEnd();
}

void drawDot(vector3D_t vc)
{
    glBegin(GL_POINTS);
        glVertex2f(vc.v[0], vc.v[1]);
    glEnd();
}

void drawLine(point3D_t p1,point3D_t p2)
{
    glBegin(GL_LINES);
        glVertex2f(p1.x, p1.y);
        glVertex2f(p2.x, p2.y);
    glEnd();
}

void drawLine(vector3D_t v1,vector3D_t v2)
{
    glBegin(GL_LINES);
        glVertex2f(v1.v[0], v1.v[1]);
        glVertex2f(v2.v[0], v2.v[1]);
    glEnd();
}

void drawLine(float x1,float y1,float x2,float y2)
{
    glBegin(GL_LINES);
        glVertex2f(x1, y1);
        glVertex2f(x2, y2);
    glEnd();
}

//n: number of points
void drawPolyline(point3D_t pnt[],int n)
{
    int i;
    glBegin(GL_LINE_STRIP);
        for (i=0;i<n;i++) {
            glVertex2f(pnt[i].x, pnt[i].y);
        }
    glEnd();
}

void drawPolyline(vector3D_t vct[],int n)
{
    int i;
    glBegin(GL_LINE_STRIP);
        for (i=0;i<n;i++) {
            glVertex2f(vct[i].v[0], vct[i].v[1]);
        }
    glEnd();
}

//n: number of vertices
void drawPolygon(point3D_t pnt[],int n)
{
    int i;
    glBegin(GL_LINE_LOOP);
        for (i=0;i<n;i++) {
            glVertex2f(pnt[i].x, pnt[i].y);
        }
    glEnd();
}

void drawPolygon(vector3D_t vct[],int n)
{
    int i;
    glBegin(GL_LINE_LOOP);
        for (i=0;i<n;i++) {
            glVertex2f(vct[i].v[0], vct[i].v[1]);
        }
    glEnd();
}

// The function fillPolygon can fills only convex polygons
//n: number of vertices
void fillPolygon(point3D_t pnt[],int n,color_t color)
{
    int i;
    setColor(color);
    glBegin(GL_POLYGON);
        for (i=0;i<n;i++) {
            glVertex2f(pnt[i].x, pnt[i].y);
        }
    glEnd();
}

void fillPolygon(vector3D_t vct[],int n,color_t color)
{
    int i;
    setColor(color);
    glBegin(GL_POLYGON);
        for (i=0;i<n;i++) {
            glVertex2f(vct[i].v[0], vct[i].v[1]);
        }
    glEnd();
}

// The function gradatePolygon can fills only convex polygons
// The vertices will be painted with corresponding given colors.
// The points inside the polygon will be painted with the mixed color.
//n: number of vertices
void gradatePolygon(point3D_t pnt[],color_t col[],int num)
{
    int i;
    glBegin(GL_POLYGON);
        for (i=0;i<num;i++) {
            setColor(col[i]);
            glVertex2f(pnt[i].x, pnt[i].y);
        }
    glEnd();
}

void gradatePolygon(vector3D_t vct[],color_t col[],int num)
{
    int i;
    glBegin(GL_POLYGON);
        for (i=0;i<num;i++) {
            setColor(col[i]);
            glVertex2f(vct[i].v[0], vct[i].v[1]);
        }
    glEnd();
}

